# Double characters
s/Ya/Я/g
s/ya/я/g
s/Yo/Ё/g
s/yo/ё/
s/Yu/Ю/g
s/yu/ю/g
s/Ts/Ц/g
s/ts/ц/g
s/Şç/Щ/g
s/şç/щ/g
# Single characters
s/A/А/g
s/a/а/g
s/Ä/Ә/g
s/ä/ә/g
s/B/Б/g
s/b/б/g
s/V/В/g
s/v/в/g
s/Ğ/Г/g
s/ğ/г/g
s/D/Д/g
s/d/д/g
s/E/Е/g
s/e/е/g
s/J/Ж/g
s/j/ж/g
s/C/Җ/g
s/c/җ/g
s/Z/З/g
s/z/з/g
s/I/И/g
s/i/и/g
s/Y/Й/g
s/y/й/g
s/K/К/g
s/k/к/g
s/Q/К/g
s/q/к/g
s/L/Л/g
s/l/л/g
s/M/М/g
s/m/м/g
s/N/Н/g
s/n/н/g
s/Ñ/Ң/g
s/ñ/ң/g
s/O/О/g
s/o/о/g
s/Ö/Ө/g
s/ö/ө/g
s/P/П/g
s/p/п/g
s/R/Р/g
s/r/р/g
s/S/С/g
s/s/с/g
s/T/Т/g
s/t/т/g
s/U/У/g
s/u/у/g
s/Ü/Ү/g
s/ü/ү/g
s/F/Ф/g
s/f/ф/g
s/X/Х/g
s/x/х/g
s/H/Һ/g
s/h/һ/g
s/Ç/Ч/g
s/ç/ч/g
s/Ş/Ш/g
s/ş/ш/g
# Note: The upper I without dot cannot be transliterated, it's already used above.
# s/I/Ы/g
s/ı/ы/g
# The following cyrillic chars are already used for "V" and "v", so not unique.
s/W/В/g
s/w/в/g
